/*
 *  Copyright (C) 2003, 2004 Laurent Sansonetti <lrz@gnome.org>
 *
 *  This file is part of Ruby/Libburn.
 *
 *  Ruby/Libburn is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  Ruby/Libburn is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Ruby/Libburn; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include "rbburn.h"

/* Module: Burn
 * The Burn module.
 */

/*
 * Class method: init
 *
 * This must be called before using any other methods in the library.
 * It may be called more than once with no effect.
 * If is possible to 'restart' the library by shutting it down and
 * re-initializing it, though there is no good reason to do that.
 *
 * Returns: true if the library was able to initialize, false if
 * initialization failed.
 */
static VALUE
rbburn_init (VALUE self)
{
	return CBOOL2RVAL (burn_initialize ());
}

/*
 * Class method: finish
 *
 * Shutdowns the library.  This should be called before exiting your
 * application. Make sure that all drives you have grabbed are released
 * before calling this.
 *
 * Returns: always nil.
 */
static VALUE
rbburn_finish (VALUE self)
{
	burn_finish ();
	return Qnil;
}

/*
 * Class method: verbosity=(level)
 * level: the verbosity level desired (0 for nothing, higher positive values
 * for more information output.
 *
 * Sets the verbosity level of the library.
 * The default value is 0, which means that nothing is output on stderr.
 * The more you increase this, the more debug output should be displayed on
 * stderr for you.
 *
 * Returns: the level parameter.
 */
static VALUE
rbburn_set_verbosity (VALUE self, int level)
{
	burn_set_verbosity (FIX2INT (level));
	return level;
}

/*
 * Class method: message
 *
 * Retrieves a message.
 *
 * Returns: a newly allocated Burn::Message or nil when there are no more
 * messages to retrieve.
 */
static VALUE
rbburn_get_message (VALUE self)
{
	return rbburn_message_make (burn_get_message ());
}

static void
init_all_subclasses (VALUE mBurn)
{
	rbburn_disc_init (mBurn);
	rbburn_drive_init (mBurn);
	rbburn_message_init (mBurn);
	rbburn_progress_init (mBurn);
	rbburn_read_opts_init (mBurn);
	rbburn_session_init (mBurn);
	rbburn_source_init (mBurn);
	rbburn_toc_entry_init (mBurn);
	rbburn_track_init (mBurn);
	rbburn_write_opts_init (mBurn);
}

/*
 * Class method: msf_to_sectors(minutes, seconds, frames)
 * minutes: the minute component.
 * seconds: the second component.
 * frames: the frame component.
 *
 * Converts a minute-second-frame (MSF) value to a sector count.
 *
 * Returns: a sector count.
 */
static VALUE
rbburn_msf_to_sectors (VALUE self, VALUE minutes, VALUE seconds, VALUE frames)
{
	return INT2FIX (burn_msf_to_sectors (FIX2INT (minutes),
					     FIX2INT (seconds),
					     FIX2INT (frames)));
}

/*
 * Class method: sectors_to_msf(lba)
 * lba: the sector count. 
 *
 * Converts a sector count to a minute-second-frame (MSF).
 *
 *	# Example:
 *	minutes, seconds, frames = Burn.sectors_to_msf(100)
 *
 * Returns: an Array of 3 elements which contains the minutes, seconds
 * and frames components.
 */
static VALUE
rbburn_sectors_to_msf (VALUE self, VALUE lba)
{
	int minutes, seconds, frames;
	VALUE arr;

	burn_sectors_to_msf (FIX2INT (lba), &minutes, &seconds, &frames);
	arr = rb_ary_new ();
	rb_ary_push (arr, INT2FIX (minutes));
	rb_ary_push (arr, INT2FIX (seconds));
	rb_ary_push (arr, INT2FIX (frames));
	return arr;
}

void
Init_libburn (void)
{
	/* create the module */
	VALUE mBurn = rb_define_module ("Burn");
	rb_define_module_function (mBurn, "init", rbburn_init, 0);
	rb_define_module_function (mBurn, "finish", rbburn_finish, 0);
	rb_define_module_function (mBurn, "verbosity=",
				   rbburn_set_verbosity, 1);
	rb_define_module_function (mBurn, "message", rbburn_get_message, 0);
	rb_define_module_function (mBurn, "msf_to_sectors",
				   rbburn_msf_to_sectors, 3);
	rb_define_module_function (mBurn, "sectors_to_msf",
				   rbburn_sectors_to_msf, 1);

	/* Constant: POS_END
	 * Identifies the "at the end" position.
	 */
	rb_define_const (mBurn, "POS_END", INT2FIX (BURN_POS_END));

	/* Constant: BLOCK_RAW0
	 * sync, headers, edc/ecc provided by lib/user.
	 */
	rb_define_const (mBurn, "BLOCK_RAW0", INT2FIX (BURN_BLOCK_RAW0));

	/* Constant: BLOCK_RAW16
	 * sync, headers, edc/ecc and p/q subs provided by lib/user.
	 */
	rb_define_const (mBurn, "BLOCK_RAW16", INT2FIX (BURN_BLOCK_RAW16));

	/* Constant: BLOCK_RAW96P
	 * sync, headers, edc/ecc and packed p-w subs provided by lib/user.
	 */
	rb_define_const (mBurn, "BLOCK_RAW96P", INT2FIX (BURN_BLOCK_RAW96P));

	/* Constant: BLOCK_RAW96R
	 * sync, headers, edc/ecc and raw p-w subs provided by lib/user.
	 */
	rb_define_const (mBurn, "BLOCK_RAW96R", INT2FIX (BURN_BLOCK_RAW96R));

	/* Constant: BLOCK_MODE1
	 * only 2048 bytes of user data provided by lib/user.
	 */
	rb_define_const (mBurn, "BLOCK_MODE1", INT2FIX (BURN_BLOCK_MODE1));
	
	/* Constant: BLOCK_MODE2R
	 * 2336 bytes of user data provided by lib/user. 
	 */
	rb_define_const (mBurn, "BLOCK_MODE2R", INT2FIX (BURN_BLOCK_MODE2R));

	/* Constant: BLOCK_MODE2_PATHETIC
	 * 2048 bytes of user data provided by lib/user subheader provided
	 * in write parameters.
	 */
	rb_define_const (mBurn, "BLOCK_MODE2_PATHETIC",
			 INT2FIX (BURN_BLOCK_MODE2_PATHETIC));

	/* Constant: BLOCK_MODE2_LAME
	 * 2048 bytes of data + 8 byte subheader provided by lib/user.
	 */
	rb_define_const (mBurn, "BLOCK_MODE2_LAME",
			 INT2FIX (BURN_BLOCK_MODE2_LAME));

	/* Constant: BLOCK_MODE2_OBSCURE
	 * 2324 bytes of data provided by lib/user subheader provided in
	 * write parameters.
	 */
	rb_define_const (mBurn, "BLOCK_MODE2_OBSCURE",
			 INT2FIX (BURN_BLOCK_MODE2_OBSCURE));
	
	/* Constant: BLOCK_MODE2_OK
	 * 2332 bytes of data supplied by lib/user. 8 bytes sub header
	 * provided in write parameters.
	 */
	rb_define_const (mBurn, "BLOCK_MODE2_OK",
			 INT2FIX (BURN_BLOCK_MODE2_OK));

	/* Constant: CDROM
	 * Session format for normal audio or data discs.
	 */
	rb_define_const (mBurn, "CDROM", INT2FIX (BURN_CDROM));

	/* Constant: CDI
	 * Session format for obsolete CD-I discs.
	 */
	rb_define_const (mBurn, "CDI", INT2FIX (BURN_CDI));

	/* Constant: CDXA
	 * Session format for CDROM-XA discs.
	 */
	rb_define_const (mBurn, "CDXA", INT2FIX (BURN_CDXA));

	/* initialize subclasses */
	init_all_subclasses (mBurn);
}
