/*
 *  Copyright (C) 2003, 2004 Laurent Sansonetti <lrz@gnome.org>
 *
 *  This file is part of Ruby/Libburn.
 *
 *  Ruby/Libburn is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  Ruby/Libburn is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Ruby/Libburn; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "rbburn.h"

/* Class: Burn::Disc
 * This class references a whole disc.
 */
static VALUE cBurnDisc;

VALUE
rbburn_disc_make (struct burn_disc *c_struct)
{
	return c_struct != NULL
		? Data_Wrap_Struct (cBurnDisc,
				    NULL,
				    burn_disc_free,
				    c_struct)
		: Qnil;
}

static struct burn_disc *
rbburn_disc_get (VALUE obj)
{
	struct burn_disc *c_struct;

	Data_Get_Struct(obj, struct burn_disc, c_struct);
	assert (c_struct != NULL);
	return c_struct;
}

/*
 * Class method: new
 *
 * Creates a new disc (for DAO recording).
 *
 * Returns: a newly allocated Burn::Disc object.
 */
static VALUE 
rbburn_disc_new (VALUE self)
{
	return rbburn_disc_make (burn_disc_create ());
}

/*
 * Method: add_session(session, pos)
 * session: a session to add to the disk (as a Burn::Session object).
 * pos: position to add at (Burn::POS_END is "at the end").
 *
 * Adds a session to a disc at a specific position, increasing the
 * sessions' reference count.
 *
 * Returns: true on success, false otherwise.
 */
static VALUE
rbburn_disc_add_session (VALUE self, VALUE session, VALUE pos)
{
	return CBOOL2RVAL (burn_disc_add_session (rbburn_disc_get (self),
						  rbburn_session_get (session),
						  FIX2INT (pos)));
}

/*
 * Method: remove_session(session)
 * session: a session to remove from the disk (as a Burn::Session object).
 *
 * Removes a session from a disc.
 *
 * Returns: true on success, false otherwise.
 */
static VALUE
rbburn_disc_remove_session (VALUE self, VALUE session)
{
	return CBOOL2RVAL (burn_disc_remove_session (
		rbburn_disc_get (self),
		rbburn_session_get (session)));
}

/*
 * Method: sessions
 *
 * Gets a list of all sessions in the disc.
 *
 * Returns: an Array of Burn::Session objects.
 */
static VALUE
rbburn_disc_get_sessions (VALUE self)
{
	struct burn_session **sessions;
	int n_sessions, i;
	VALUE arr;
	
	sessions = burn_disc_get_sessions (rbburn_disc_get (self), &n_sessions);
	arr = rb_ary_new ();
	for (i = 0; i < n_sessions; i++) {
		rb_ary_push (arr, rbburn_session_make (sessions[i]));
	}
	return arr;
}

/*
 * Method: each_session { |session| ... }
 *
 * Calls the block for each session in the disc, passing a reference
 * to a Burn::Session object as parameter.
 *
 * Returns: always nil.
 */
static VALUE
rbburn_disc_each_session (VALUE self)
{
	rb_ary_each (rbburn_disc_get_sessions (self));
	return Qnil;
}

void
rbburn_disc_init (VALUE mBurn)
{
	VALUE c = rb_define_class_under (mBurn, "Disc", rb_cObject);
	rb_define_singleton_method (c, "new", rbburn_disc_new, 0);
	rb_define_method (c, "add_session", rbburn_disc_add_session, 2);
	rb_define_method (c, "remove_session", rbburn_disc_remove_session, 1);
	rb_define_method (c, "sessions", rbburn_disc_get_sessions, 0);
	rb_define_method (c, "each_session", rbburn_disc_each_session, 0);

	/* Constant: STATUS_UNREADY
	 * The current status is not yet known.
	 */
	rb_define_const (c, "STATUS_UNREADY", INT2FIX (BURN_DISC_UNREADY));

	/* Constant: STATUS_BLANK
	 * The drive holds a blank disc.
	 */
	rb_define_const (c, "STATUS_BLANK", INT2FIX (BURN_DISC_BLANK));

	/* Constant: STATUS_EMPTY
	 * There is no disc at all in the drive.
	 */
	rb_define_const (c, "STATUS_EMPTY", INT2FIX (BURN_DISC_EMPTY));

	/* Constant: STATUS_APPENDABLE
	 * There is an incomplete disc in the drive.
	 */
	rb_define_const (c, "STATUS_APPENDABLE",
			 INT2FIX (BURN_DISC_APPENDABLE));

	/* Constant: STATUS_FULL
	 * There is a disc with data on it in the drive.
	 */
	rb_define_const (c, "STATUS_FULL", INT2FIX (BURN_DISC_FULL));
	cBurnDisc = c;
}
