/*
 *  Copyright (C) 2003, 2004 Laurent Sansonetti <lrz@gnome.org>
 *
 *  This file is part of Ruby/Libburn.
 *
 *  Ruby/Libburn is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  Ruby/Libburn is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Ruby/Libburn; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "rbburn.h"

/* Class: Burn::Drive
 * This class references a physical drive in the system.
 */
static VALUE cBurnDrive;

static VALUE
rbburn_drive_make (struct burn_drive_info *c_struct)
{
	return c_struct != NULL
		? Data_Wrap_Struct (cBurnDrive,
				    NULL,
				    free,
				    c_struct)
		: Qnil;
}

struct burn_drive_info *
rbburn_drive_get (VALUE obj)
{
	struct burn_drive_info *c_struct;

	Data_Get_Struct(obj, struct burn_drive_info, c_struct);
	assert (c_struct != NULL);
	return c_struct;
}

/*
 * Method: grab!(load)
 * load: true to make the drive attempt to load a disc (close its tray door,
 * etc).
 *
 * Grabs a drive. This must be done before the drive can be used (for
 * reading, writing, etc). It may be neccesary to call this method more than
 * once to grab a drive.
 *
 * Returns: true is the drive has been grabbed, false otherwise.
 */
static VALUE
rbburn_drive_grab (VALUE self, VALUE load)
{
	struct burn_drive_info *info = rbburn_drive_get (self);	
	return CBOOL2RVAL (burn_drive_grab (info->drive, RVAL2CBOOL (load)));
}

/*
 * Method: release!(eject)
 * eject: true to make the drive eject the disc in it.
 *
 * Releases a drive. This should not be done until the drive is no longer
 * busy (see Burn::Drive#status).
 *
 * Returns: self.
 */
static VALUE
rbburn_drive_release (VALUE self, VALUE eject)
{
	struct burn_drive_info *info = rbburn_drive_get (self);	
	burn_drive_release (info->drive, RVAL2CBOOL (eject));
	return self;
}

/*
 * Method: disc_status
 *
 * Retrieves what kind of disc a drive is holding. This method may need to be
 * called more than once to get a proper status from it.
 *
 * Returns: a disc status (either Burn::Disc::STATUS_APPENDABLE, 
 * Burn::Disc::STATUS_BLANK, Burn::Disc::STATUS_EMPTY,
 * Burn::Disc::STATUS_UNREADY, or Burn::Disc::STATUS_FULL).
 */
static VALUE
rbburn_drive_get_disc_status (VALUE self)
{
	struct burn_drive_info *info = rbburn_drive_get (self);	
	return INT2FIX (burn_disc_get_status (info->drive));
}

/*
 * Method: set_speed(read, write)
 * read: read speed in k/s (0 is max).
 * write: write speed in k/s (0 is max).
 *
 * Sets drive read and write speed.
 *
 * Returns: self.
 */
static VALUE
rbburn_drive_set_speed (VALUE self, VALUE read, VALUE write)
{
	struct burn_drive_info *info = rbburn_drive_get (self);	
	burn_drive_set_speed (info->drive, FIX2INT (read), FIX2INT (write));
	return self;
}

/* Method: read_speed
 * Returns: maximum read speed for a drive in k/s.
 */
static VALUE
rbburn_drive_get_read_speed (VALUE self)
{
	struct burn_drive_info *info = rbburn_drive_get (self);	
	return INT2FIX (burn_drive_get_read_speed (info->drive));
}

/* Method: write_speed
 * Returns: maximum write speed for a drive in k/s.
 */
static VALUE
rbburn_drive_get_write_speed (VALUE self)
{
	struct burn_drive_info *info = rbburn_drive_get (self);	
	return INT2FIX (burn_drive_get_write_speed (info->drive));
}

/*
 * Method: disc_erasable?
 *
 * Checks whether a disc can be erased or not.
 *
 * Returns: true if erasable, false if not.
 */
static VALUE
rbburn_drive_erasable (VALUE self)
{
	struct burn_drive_info *info = rbburn_drive_get (self);	
	return CBOOL2RVAL (burn_disc_erasable (info->drive));
}

/*
 * Method: erase_disc!(fast)
 * fast: true to do a fast erase, where only the disc's headers are erased;
 * false to erase the entire disc.
 * 
 * Erases a disc in the drive. The drive must be grabbed successfully BEFORE
 * calling this method. Always ensure that the drive reports a status of
 * Burn::Disc::STATUS_FULL before calling this function. An erase operation
 * is not cancellable, as control of the operation is passed wholly to the
 * drive and there is no way to interrupt it safely.
 *
 * Returns: self.
 */
static VALUE
rbburn_drive_erase_disc (VALUE self, VALUE fast)
{
	struct burn_drive_info *info = rbburn_drive_get (self);	
	burn_disc_erase (info->drive, RVAL2CBOOL (fast));
	return self;
}

/*
 * Method: cancel!
 *
 * Cancels an operation on a drive.
 * This will only work when the drive's busy state is
 * Burn::Drive::STATUS_READING or Burn::Drive::STATUS_WRITING.
 *
 * Returns: self.
 */
static VALUE
rbburn_drive_cancel (VALUE self)
{
	struct burn_drive_info *info = rbburn_drive_get (self);	
	burn_drive_cancel (info->drive);
	return self;
}

/*
 * Method: status
 *
 * Gets the status of the drive.
 *
 * Returns: a drive status (either Burn::Drive::STATUS_IDLE, 
 * Burn::Drive::STATUS_SPAWNING, Burn::Drive::STATUS_READING,
 * Burn::Drive::STATUS_WRITING, Burn::Drive::STATUS_WRITING_LEADIN,
 * Burn::Drive::STATUS_WRITING_LEADOUT, Burn::Drive::STATUS_ERASING or
 * Burn::Drive::STATUS_GRABBING).
 */
static VALUE
rbburn_drive_get_status (VALUE self)
{
	struct burn_drive_info *info = rbburn_drive_get (self);	
	return INT2FIX (burn_drive_get_status (info->drive, NULL));
}

/* Method: progress
 * Returns: the progres of the current operation on the drive, as a newly
 * allocated Burn::Progress object.
 */
static VALUE
rbburn_drive_get_progress (VALUE self)
{
	struct burn_progress *progress;
	struct burn_drive_info *info;
	
	info = rbburn_drive_get (self);
	progress = ALLOC (struct burn_progress);
	burn_drive_get_status (info->drive, progress);

	return rbburn_progress_make (progress);
}

/* Method: disc
 * Returns: the drive's disc, as a newly allocated Burn::Disc object.
 */
static VALUE
rbburn_drive_get_disc (VALUE self)
{
	struct burn_drive_info *info = rbburn_drive_get (self);	
	return burn_disc_get_status (info->drive) != BURN_DISC_EMPTY
		? rbburn_disc_make (burn_drive_get_disc (info->drive))
		: Qnil;
}

/*
 * Class method: scan
 *
 * Scans for drives.
 *
 *	# Example: prints name and location of all drives 
 *	Burn::Drive.scan.each do |drive|
 *		puts drive.product + " on " + drive.location
 *	end
 *
 * Returns: a list of drives (cdroms/burners), as an Array of Burn::Drive
 * objects.
 */
static VALUE
rbburn_drive_scan (VALUE self)
{
	static struct burn_drive_info *infos = NULL;
	VALUE drives;
	int n_drives;
	int status;
	int i;

	if (infos != NULL)
		burn_drive_info_free (infos);

	n_drives = 0;
	do {
		status = burn_drive_scan (&infos, &n_drives);
	}
	while (status == 0);

	drives = rb_ary_new ();
	for (i = 0; i < n_drives; i++) {
		assert (&infos[i] != NULL);
		rb_ary_push (drives, rbburn_drive_make (&infos[i]));
	}
	return drives;
}

/* Method: vendor
 * Returns: the name of the vendor of the drive.
 */
static VALUE
rbburn_drive_get_vendor (VALUE self)
{
	struct burn_drive_info *info = rbburn_drive_get (self);	
	return CSTR2RVAL (info->vendor);
}

/* Method: product
 * Returns: the name of the drive.
 */
static VALUE
rbburn_drive_get_product (VALUE self)
{
	struct burn_drive_info *info = rbburn_drive_get (self);	
	return CSTR2RVAL (info->product);
}

/* Method: location
 * Returns: the location of the drive in the filesystem.
 */
static VALUE
rbburn_drive_get_location (VALUE self)
{
	struct burn_drive_info *info = rbburn_drive_get (self);	
	return CSTR2RVAL (info->location);
}

/* Method: can_read_dvdram?
 * Returns: whether the drive can read DVD-RAM discs.
 */
static VALUE
rbburn_drive_read_dvdram (VALUE self)
{
	struct burn_drive_info *info = rbburn_drive_get (self);	
	return CBOOL2RVAL (info->read_dvdram);
}

/* Method: can_read_dvdr?
 * Returns: whether the drive can read DVD-R discs.
 */
static VALUE
rbburn_drive_read_dvdr (VALUE self)
{
	struct burn_drive_info *info = rbburn_drive_get (self);	
	return CBOOL2RVAL (info->read_dvdr);
}

/* Method: can_read_dvdrom?
 * Returns: whether the drive can read DVD-ROM discs.
 */
static VALUE
rbburn_drive_read_dvdrom (VALUE self)
{
	struct burn_drive_info *info = rbburn_drive_get (self);	
	return CBOOL2RVAL (info->read_dvdrom);
}

/* Method: can_read_cdr?
 * Returns: whether the drive can read CD-R discs.
 */
static VALUE
rbburn_drive_read_cdr (VALUE self)
{
	struct burn_drive_info *info = rbburn_drive_get (self);	
	return CBOOL2RVAL (info->read_cdr);
}

/* Method: can_read_cdrw?
 * Returns: whether the drive can read CD-RW discs.
 */
static VALUE
rbburn_drive_read_cdrw (VALUE self)
{
	struct burn_drive_info *info = rbburn_drive_get (self);	
	return CBOOL2RVAL (info->read_cdrw);
}

/* Method: can_write_dvdram?
 * Returns: whether the drive can write DVD-RAM discs.
 */
static VALUE
rbburn_drive_write_dvdram (VALUE self)
{
	struct burn_drive_info *info = rbburn_drive_get (self);	
	return CBOOL2RVAL (info->write_dvdram);
}

/* Method: can_write_dvdr?
 * Returns: whether the drive can write DVD-R discs.
 */
static VALUE
rbburn_drive_write_dvdr (VALUE self)
{
	struct burn_drive_info *info = rbburn_drive_get (self);	
	return CBOOL2RVAL (info->write_dvdr);
}

/* Method: can_write_cdr?
 * Returns: whether the drive can write CD-R discs.
 */
static VALUE
rbburn_drive_write_cdr (VALUE self)
{
	struct burn_drive_info *info = rbburn_drive_get (self);	
	return CBOOL2RVAL (info->write_cdr);
}

/* Method: can_write_cdrw?
 * Returns: whether the drive can write CD-RW discs.
 */
static VALUE
rbburn_drive_write_cdrw (VALUE self)
{
	struct burn_drive_info *info = rbburn_drive_get (self);	
	return CBOOL2RVAL (info->write_cdrw);
}

/* Method: can_write_simulate?
 * Returns: whether the drive can simulate a write.
 */
static VALUE
rbburn_drive_write_simulate (VALUE self)
{
	struct burn_drive_info *info = rbburn_drive_get (self);	
	return CBOOL2RVAL (info->write_simulate);
}

/* Method: can_report_c2_errors?
 * Returns: whether the drive can report C2 errors.
 */
static VALUE
rbburn_drive_c2_errors (VALUE self)
{
	struct burn_drive_info *info = rbburn_drive_get (self);	
	return CBOOL2RVAL (info->c2_errors);
}

/* Method: buffer_size
 * Returns: the size of the drive's buffer (in kilobytes).
 */
static VALUE
rbburn_drive_get_buffer_size (VALUE self)
{
	struct burn_drive_info *info = rbburn_drive_get (self);	
	return INT2FIX (info->buffer_size);
}

/*
 * Method: tao_block_types
 *
 * Returns: the supported block types in TAO mode (either Burn::BLOCK_RAW0,
 * Burn::BLOCK_RAW16, Burn::BLOCK_RAW96P, Burn::BLOCK_RAW96R,
 * Burn::BLOCK_MODE1, Burn::BLOCK_MODE2_PATHETIC, Burn::BLOCK_MODE2_LAME,
 * Burn::BLOCK_MODE2_OBSCURE, Burn::BLOCK_MODE2_OK, Burn::BLOCK_MODE2R).
 */
static VALUE
rbburn_drive_get_tao_block_types (VALUE self)
{
	struct burn_drive_info *info = rbburn_drive_get (self);	
	return INT2FIX (info->tao_block_types);
}

/*
 * Method: sao_block_types
 *
 * Returns: the supported block types in SAO mode (either Burn::BLOCK_RAW0,
 * Burn::BLOCK_RAW16, Burn::BLOCK_RAW96P, Burn::BLOCK_RAW96R,
 * Burn::BLOCK_MODE1, Burn::BLOCK_MODE2_PATHETIC, Burn::BLOCK_MODE2_LAME,
 * Burn::BLOCK_MODE2_OBSCURE, Burn::BLOCK_MODE2_OK, Burn::BLOCK_MODE2R).
 */
static VALUE
rbburn_drive_get_sao_block_types (VALUE self)
{
	struct burn_drive_info *info = rbburn_drive_get (self);	
	return INT2FIX (info->sao_block_types);
}

/*
 * Method: raw_block_types
 *
 * Returns: the supported block types in raw mode (either Burn::BLOCK_RAW0,
 * Burn::BLOCK_RAW16, Burn::BLOCK_RAW96P, Burn::BLOCK_RAW96R,
 * Burn::BLOCK_MODE1, Burn::BLOCK_MODE2_PATHETIC, Burn::BLOCK_MODE2_LAME,
 * Burn::BLOCK_MODE2_OBSCURE, Burn::BLOCK_MODE2_OK, Burn::BLOCK_MODE2R).
 */
static VALUE
rbburn_drive_get_raw_block_types (VALUE self)
{
	struct burn_drive_info *info = rbburn_drive_get (self);	
	return INT2FIX (info->raw_block_types);
}

/*
 * Method: packet_block_types
 *
 * Returns: the supported block types in packet mode (either Burn::BLOCK_RAW0,
 * Burn::BLOCK_RAW16, Burn::BLOCK_RAW96P, Burn::BLOCK_RAW96R,
 * Burn::BLOCK_MODE1, Burn::BLOCK_MODE2_PATHETIC, Burn::BLOCK_MODE2_LAME,
 * Burn::BLOCK_MODE2_OBSCURE, Burn::BLOCK_MODE2_OK, Burn::BLOCK_MODE2R).
 */
static VALUE
rbburn_drive_get_packet_block_types (VALUE self)
{
	struct burn_drive_info *info = rbburn_drive_get (self);	
	return INT2FIX (info->packet_block_types);
}

void
rbburn_drive_init (VALUE mBurn)
{
	VALUE c = rb_define_class_under (mBurn, "Drive", rb_cObject);
	rb_undef_method (CLASS_OF (c), "new");

	/* burn_drive interface */
	rb_define_method (c, "grab!", rbburn_drive_grab, 1);
	rb_define_method (c, "release!", rbburn_drive_release, 1);
	rb_define_method (c, "disc_status", rbburn_drive_get_disc_status, 0);
	rb_define_method (c, "disc_erasable?", rbburn_drive_erasable, 0);
	rb_define_method (c, "erase_disc!", rbburn_drive_erase_disc, 1);
	rb_define_method (c, "cancel!", rbburn_drive_cancel, 1);
	rb_define_method (c, "status", rbburn_drive_get_status, 0);
	rb_define_method (c, "progress", rbburn_drive_get_progress, 0);
	rb_define_method (c, "disc", rbburn_drive_get_disc, 0);
	rb_define_method (c, "set_speed", rbburn_drive_set_speed, 2);
	rb_define_method (c, "read_speed", rbburn_drive_get_read_speed, 0);
	rb_define_method (c, "write_speed", rbburn_drive_get_write_speed, 0);
	
	/* burn_drive_info interface */
	rb_define_singleton_method (c, "scan", rbburn_drive_scan, 0);
	rb_define_method (c, "vendor", rbburn_drive_get_vendor, 0);
	rb_define_method (c, "product", rbburn_drive_get_product, 0);
	rb_define_method (c, "location", rbburn_drive_get_location, 0);
	rb_define_method (c, "can_read_dvdram?", rbburn_drive_read_dvdram, 0);
	rb_define_method (c, "can_read_dvdr?", rbburn_drive_read_dvdr, 0);
	rb_define_method (c, "can_read_dvdrom?", rbburn_drive_read_dvdrom, 0);
	rb_define_method (c, "can_read_cdr?", rbburn_drive_read_cdr, 0);
	rb_define_method (c, "can_read_cdrw?", rbburn_drive_read_cdrw, 0);
	
	rb_define_method (
		c,"can_write_dvdram?", rbburn_drive_write_dvdram, 0);
	rb_define_method (c, "can_write_dvdr?", rbburn_drive_write_dvdr, 0);
	rb_define_method (c, "can_write_cdr?", rbburn_drive_write_cdr, 0);
	rb_define_method (c, "can_write_cdrw?", rbburn_drive_write_cdrw, 0);
	rb_define_method (
		c, "can_write_simulate?", rbburn_drive_write_simulate, 0);

	rb_define_method (
		c,"can_report_c2_errors?", rbburn_drive_c2_errors, 0);

	rb_define_method (c, "buffer_size", rbburn_drive_get_buffer_size, 0);
	rb_define_method (
		c, "tao_block_types", rbburn_drive_get_tao_block_types, 0);
	rb_define_method (
		c, "sao_block_types", rbburn_drive_get_sao_block_types, 0);
	rb_define_method (
		c, "raw_block_types", rbburn_drive_get_raw_block_types, 0);
	rb_define_method (
		c, "packet_block_types", rbburn_drive_get_packet_block_types,
		0);

	/* Constant: STATUS_IDLE
	 * The drive is not in an operation.
	 */
	rb_define_const (c, "STATUS_IDLE", INT2FIX (BURN_DRIVE_IDLE));

	/* Constant: STATUS_SPAWNING
	 * The library is spawning the processes to handle a pending
	 * operation (A read/write/etc is about to start but hasn't quite
	 * yet)
	 */
	rb_define_const (c, "STATUS_SPAWNING", INT2FIX (BURN_DRIVE_SPAWNING));

	/* Constant: STATUS_READING
	 * The drive is reading data from a disc.
	 */
	rb_define_const (c, "STATUS_READING", INT2FIX (BURN_DRIVE_READING));

	/* Constant: STATUS_WRITING
	 * The drive is writing data to a disc.
	 */
	rb_define_const (c, "STATUS_WRITING", INT2FIX (BURN_DRIVE_WRITING));

	/* Constant: STATUS_WRITING_LEADIN
	 * The drive is writing Lead-In.
	 */
	rb_define_const (c, "STATUS_WRITING_LEADIN",
			 INT2FIX (BURN_DRIVE_WRITING_LEADIN));

	/* Constant: STATUS_WRITING_LEADOUT
	 * The drive is writing Lead-Out.
	 */
	rb_define_const (c, "STATUS_WRITING_LEADOUT",
			 INT2FIX (BURN_DRIVE_WRITING_LEADOUT));

	/* Constant: STATUS_ERASING
	 * The drive is erasing a disc.
	 */
	rb_define_const (c, "STATUS_ERASING", INT2FIX (BURN_DRIVE_ERASING));

	/* Constant: STATUS_GRABBING
	 * The drive is being grabbed.
	 */
	rb_define_const (c, "STATUS_GRABBING", INT2FIX (BURN_DRIVE_GRABBING));
	cBurnDrive = c;
}
