/*
 *  Copyright (C) 2003, 2004 Laurent Sansonetti <lrz@gnome.org>
 *
 *  This file is part of Ruby/Libburn.
 *
 *  Ruby/Libburn is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  Ruby/Libburn is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Ruby/Libburn; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "rbburn.h"

/* Class: Burn::ReadOptions
 * Options relative to reading operations.
 */
static VALUE cBurnReadOpts;

static VALUE
rbburn_read_opts_make (struct burn_read_opts *c_struct)
{
	return c_struct != NULL
		? Data_Wrap_Struct (cBurnReadOpts,
				    NULL,
				    burn_read_opts_free,
				    c_struct)
		: Qnil;
}

struct burn_read_opts *
rbburn_read_opts_get (VALUE obj)
{
	struct burn_read_opts *c_struct;

	Data_Get_Struct (obj, struct burn_read_opts, c_struct);
	assert (c_struct != NULL);
	return c_struct;
}

/*
 * Class method: new(drive)
 * drive: the drive to read from (as a Burn::Drive).
 *
 * Creates a Burn::ReadOptions for reading the specified drive.
 *
 * Returns: a newly allocated Burn::ReadOptions object.
 */
static VALUE
rbburn_read_opts_new (VALUE self, VALUE drive)
{
	struct burn_drive_info *info = rbburn_drive_get (drive);
	return rbburn_read_opts_make (burn_read_opts_new (info->drive));
}

/* Method: raw=(state)
 * state: if true, reading will be done in raw mode, so that everything in
 * the data tracks on the disc is read, including headers.
 *
 * Sets whether to read in raw mode or not.
 *
 * Returns: the state. 
 */
static VALUE
rbburn_read_opts_set_raw (VALUE self, VALUE state)
{
	burn_read_opts_set_raw (rbburn_read_opts_get (self),
				RVAL2CBOOL (state));
	return state;
}

/*
 * Method: c2_errors=(state)
 * state: if true, report C2 errors.
 *
 * Sets whether to report C2 errors or not.
 *
 * Returns: the state.
 */
static VALUE
rbburn_read_opts_set_c2errors (VALUE self, VALUE state)
{
	burn_read_opts_set_c2errors (rbburn_read_opts_get (self),
				     RVAL2CBOOL (state));
	return state;
}

/*
 * Method: read_subcodes_audio=(state)
 * state: if true, read subcodes from audio tracks on the disc.
 *
 * Sets whether to read subcodes from audio tracks or not.
 *
 * Returns: the state.
 */
static VALUE
rbburn_read_opts_read_subcodes_audio (VALUE self, VALUE state)
{
	burn_read_opts_read_subcodes_audio (rbburn_read_opts_get (self),
					    RVAL2CBOOL (state));
	return state;
}

/*
 * Method: read_subcodes_data=(state)
 * state: if true, read subcodes from data tracks on the disc.
 *
 * Sets whether to read subcodes from data tracks or not.
 *
 * Returns: the state.
 */
static VALUE
rbburn_read_opts_read_subcodes_data (VALUE self, VALUE state)
{
	burn_read_opts_read_subcodes_data (rbburn_read_opts_get (self),
					   RVAL2CBOOL (state));
	return state;
}

/*
 * Method: hardware_error_recovery=(state)
 * state: if true, attempt to recover errors if possible.
 * 
 * Sets whether to recover errors if possible.
 *
 * Returns: the state.
 */
static VALUE
rbburn_read_opts_set_hardware_error_recovery (VALUE self, VALUE state)
{
	burn_read_opts_set_hardware_error_recovery (
		rbburn_read_opts_get (self), RVAL2CBOOL (state));
	return state;
}

/*
 * Method: report_recovered_errors=(state)
 * state: if true, recovered errors will be reported.
 *
 * Sets whether to report recovered errors or not.
 *
 * Returns: the state.
 */
static VALUE
rbburn_read_opts_report_recovered_errors (VALUE self, VALUE state)
{
	burn_read_opts_report_recovered_errors (rbburn_read_opts_get (self),
						RVAL2CBOOL (state));
	return state;
}

/*
 * Method: transfer_damaged_blocks=(state)
 * state: if true, blocks with unrecoverable errors will still be read.
 *
 * Sets whether blocks with unrecoverable errors should be read or not.
 * 
 * Returns: the state.
 */
static VALUE
rbburn_read_opts_transfer_damaged_blocks (VALUE self, VALUE state)
{
	burn_read_opts_transfer_damaged_blocks (rbburn_read_opts_get (self),
						RVAL2CBOOL (state));
	return state;
}

/*
 * Method: hardware_error_retries=(state)
 * state: the number of retries to attempt when correcting an error.
 *
 * Sets the number of retries to attempt when trying to correct an error.
 *
 * Returns: the state.
 */
static VALUE
rbburn_read_opts_set_hardware_error_retries (VALUE self, VALUE state)
{
	burn_read_opts_set_hardware_error_retries (
		rbburn_read_opts_get (self), FIX2INT (state));
	return state;
}

void
rbburn_read_opts_init (VALUE mBurn)
{
	VALUE c = rb_define_class_under (mBurn, "ReadOptions", rb_cObject);
	rb_define_singleton_method (c, "new", rbburn_read_opts_new, 1);
	rb_define_method (c, "raw=", rbburn_read_opts_set_raw, 1);
	rb_define_method (c, "c2_errors=", rbburn_read_opts_set_c2errors, 1);
	rb_define_method (c, "read_subcodes_audio=",
			  rbburn_read_opts_read_subcodes_audio, 1);
	rb_define_method (c, "read_subcodes_data=",
			  rbburn_read_opts_read_subcodes_data, 1);
	rb_define_method (c, "hardware_error_recovery=",
			  rbburn_read_opts_set_hardware_error_recovery, 1);
	rb_define_method (c, "report_recovered_errors=",
			  rbburn_read_opts_report_recovered_errors, 1);
	rb_define_method (c, "transfer_damaged_blocks=",
			  rbburn_read_opts_transfer_damaged_blocks, 1);
	rb_define_method (c, "hardware_error_retries=",
			  rbburn_read_opts_set_hardware_error_retries, 1);
	cBurnReadOpts = c;
}
