/*
 *  Copyright (C) 2003, 2004 Laurent Sansonetti <lrz@gnome.org>
 *
 *  This file is part of Ruby/Libburn.
 *
 *  Ruby/Libburn is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  Ruby/Libburn is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Ruby/Libburn; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "rbburn.h"

/* Class: Burn::Session
 * This class references a single session on a disc.
 */
static VALUE cBurnSession;

VALUE
rbburn_session_make (struct burn_session *c_struct)
{
	return c_struct != NULL
		? Data_Wrap_Struct (cBurnSession,
				    NULL,
				    burn_session_free,
				    c_struct)
		: Qnil;
}

struct burn_session *
rbburn_session_get (VALUE obj)
{
	struct burn_session *c_struct;

	Data_Get_Struct(obj, struct burn_session, c_struct);
	assert (c_struct != NULL);
	return c_struct;
}

/*
 * Class method: new
 *
 * Creates a new session (For SAO at once recording, or to be added to a
 * disc for DAO).
 *
 * Returns: a newly allocated Burn::Session object.
 */
static VALUE 
rbburn_session_new (VALUE self)
{
	return rbburn_session_make (burn_session_create ());
}

/*
 * Method: add_track(track, pos)
 * track: the track to insert in the session (as a Burn::Track object).
 * pos: position to add at (Burn::POS_END is "at the end").
 *
 * Adds a track to the session at specified position.
 *
 * Returns: true on success, false otherwise.
 */
static VALUE 
rbburn_session_add_track (VALUE self, VALUE track, VALUE pos)
{
	return CBOOL2RVAL (burn_session_add_track (rbburn_session_get (self),
						   rbburn_track_get (track),
						   FIX2INT (pos)));
}

/*
 * Method: remove_track(track)
 * track: the track to remove from the session (as a Burn::Track object).
 *
 * Removes a track from the session.
 *
 * Returns: true on success, false otherwise.
 */
static VALUE 
rbburn_session_remove_track (VALUE self, VALUE track)
{
	return CBOOL2RVAL (burn_session_remove_track (
		rbburn_session_get (self),
		rbburn_track_get (track)));
}

/*
 * Method: hide_first_track=(state)
 * state: true to enable hiding, false to disable.
 *
 * Hides the first track in the "pre gap" of the disc.
 *
 * Returns: the state.
 */
static VALUE
rbburn_session_set_hide_first_track (VALUE self, VALUE mode)
{
	burn_session_hide_first_track (rbburn_session_get (self),
				       RVAL2CBOOL (mode));
	return mode;
}

/*
 * Method: hide_first_track?
 *
 * Returns: whether the first track is hidden or not in the "pre gap"
 * of the disc.
 */
static VALUE
rbburn_session_get_hide_first_track (VALUE self)
{
	struct burn_session *session = rbburn_session_get (self);
	return CBOOL2RVAL (burn_session_get_hidefirst (session));
}

/*
 * Method: tracks
 *
 * Gets a list of all tracks in the session.
 *
 * Returns: an Array of Burn::Track objects.
 */
static VALUE
rbburn_session_get_tracks (VALUE self)
{
	struct burn_track **tracks;
	int n_tracks, i;
	VALUE arr;
	
	tracks = burn_session_get_tracks (rbburn_session_get (self), &n_tracks);
	arr = rb_ary_new ();
	for (i = 0; i < n_tracks; i++) {
		rb_ary_push (arr, rbburn_track_make (tracks[i]));
	}
	return arr;
}

/*
 * Method: each_track { |track| ... }
 *
 * Calls the block for each track in the session, passing a reference to a
 * Burn::Track object as parameter.
 *
 * Returns: always nil.
 */
static VALUE
rbburn_session_each_track (VALUE self)
{
	rb_ary_each (rbburn_session_get_tracks (self));
	return Qnil;
}

/*
 *
 *
 */
static VALUE
rbburn_session_get_leadout_entry (VALUE self)
{
	struct burn_toc_entry entry;
	burn_session_get_leadout_entry (rbburn_session_get (self), &entry);
	return rbburn_toc_entry_make (&entry);
}

void
rbburn_session_init (VALUE mBurn)
{
	VALUE c = rb_define_class_under (mBurn, "Session", rb_cObject);
	rb_define_singleton_method (c, "new", rbburn_session_new, 0);
	rb_define_method (c, "add_track", rbburn_session_add_track, 2);
	rb_define_method (c, "remove_track", rbburn_session_remove_track, 1);
	rb_define_method (c, "hide_first_track=",
			  rbburn_session_set_hide_first_track, 1);
	rb_define_method (c, "hide_first_track?",
			  rbburn_session_get_hide_first_track, 1);
	rb_define_method (c, "tracks", rbburn_session_get_tracks, 0); 
	rb_define_method (c, "each_track", rbburn_session_each_track, 0); 
	rb_define_method (c, "leadout_entry",
			  rbburn_session_get_leadout_entry, 0);
	cBurnSession = c;
}
