/*
 *  Copyright (C) 2003, 2004 Laurent Sansonetti <lrz@gnome.org>
 *
 *  This file is part of Ruby/Libburn.
 *
 *  Ruby/Libburn is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  Ruby/Libburn is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Ruby/Libburn; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "rbburn.h"

/* Class: Burn::TocEntry
 * Information about a track on a disc - this is from the q sub channel of the
 * lead-in area of a disc.  The documentation here is very terse.
 * See a document such as mmc3 for proper information.
 */
static VALUE cBurnTocEntry;

VALUE
rbburn_toc_entry_make (struct burn_toc_entry *c_struct)
{
	return c_struct != NULL
		? Data_Wrap_Struct (cBurnTocEntry,
				    NULL,
				    NULL,
				    c_struct)
		: Qnil;
}

struct burn_toc_entry *
rbburn_toc_entry_get (VALUE obj)
{
	struct burn_toc_entry *c_struct;

	Data_Get_Struct (obj, struct burn_toc_entry, c_struct);
	assert (c_struct != NULL);
	return c_struct;
}

/*
 * Class method: new
 *
 * Creates a new TOC entry.
 *
 * Returns: a newly allocated Burn::TocEntry object.
 */
static VALUE
rbburn_toc_entry_new (VALUE self)
{
	struct burn_toc_entry *toc;

	toc = ALLOC (struct burn_toc_entry);
	memset (toc, 0, sizeof (struct burn_toc_entry));
	return rbburn_toc_entry_make (toc);
}

/*
 * Method: session
 *
 * Returns: the 'session' property of the TOC entry.
 */
static VALUE
rbburn_toc_entry_get_session (VALUE self)
{
	struct burn_toc_entry *toc = rbburn_toc_entry_get (self);
	return INT2FIX (toc->session);	
}

/*
 * Method: session=(session)
 * session: the 'session' property to set.
 *
 * Sets the 'session' property of the TOC entry.
 *
 * Returns: the 'session' property.
 */
static VALUE
rbburn_toc_entry_set_session (VALUE self, VALUE session)
{
	struct burn_toc_entry *toc = rbburn_toc_entry_get (self);
	toc->session = FIX2INT (session);
	return session;
}

/*
 * Method: adr
 *
 * Returns: the 'adr' property of the TOC entry.
 */
static VALUE
rbburn_toc_entry_get_adr (VALUE self)
{
	struct burn_toc_entry *toc = rbburn_toc_entry_get (self);
	return INT2FIX (toc->adr);	
}

/*
 * Method: adr=(adr)
 * adr: the 'adr' property to set.
 *
 * Sets the 'adr' property of the TOC entry.
 *
 * Returns: the 'adr' property.
 */
static VALUE
rbburn_toc_entry_set_adr (VALUE self, VALUE adr)
{
	struct burn_toc_entry *toc = rbburn_toc_entry_get (self);
	toc->adr = FIX2INT (adr);
	return adr;
}

/*
 * Method: control
 *
 * Returns: the 'control' property of the TOC entry.
 */
static VALUE
rbburn_toc_entry_get_control (VALUE self)
{
	struct burn_toc_entry *toc = rbburn_toc_entry_get (self);
	return INT2FIX (toc->control);	
}

/*
 * Method: control=(control)
 * control: the 'control' property to set.
 *
 * Sets the 'control' property of the TOC entry.
 *
 * Returns: the 'control' property.
 */
static VALUE
rbburn_toc_entry_set_control (VALUE self, VALUE control)
{
	struct burn_toc_entry *toc = rbburn_toc_entry_get (self);
	toc->control = FIX2INT (control);
	return control;
}

/*
 * Method: tno
 *
 * Returns: the 'tno' property of the TOC entry.
 */
static VALUE
rbburn_toc_entry_get_tno (VALUE self)
{
	struct burn_toc_entry *toc = rbburn_toc_entry_get (self);
	return INT2FIX (toc->tno);	
}

/*
 * Method: tno=(tno)
 * tno: the 'tno' property to set.
 *
 * Sets the 'tno' property of the TOC entry.
 *
 * Returns: the 'tno' property.
 */
static VALUE
rbburn_toc_entry_set_tno (VALUE self, VALUE tno)
{
	struct burn_toc_entry *toc = rbburn_toc_entry_get (self);
	toc->tno = FIX2INT (tno);
	return tno;
}

/*
 * Method: point
 *
 * Returns: the 'point' property of the TOC entry.
 */
static VALUE
rbburn_toc_entry_get_point (VALUE self)
{
	struct burn_toc_entry *toc = rbburn_toc_entry_get (self);
	return INT2FIX (toc->point);	
}

/*
 * Method: point=(point)
 * point: the 'point' property to set.
 *
 * Sets the 'point' property of the TOC entry.
 *
 * Returns: the 'point' property.
 */
static VALUE
rbburn_toc_entry_set_point (VALUE self, VALUE point)
{
	struct burn_toc_entry *toc = rbburn_toc_entry_get (self);
	toc->point = FIX2INT (point);
	return point;
}

/*
 * Method: min
 *
 * Returns: the 'min' property of the TOC entry.
 */
static VALUE
rbburn_toc_entry_get_min (VALUE self)
{
	struct burn_toc_entry *toc = rbburn_toc_entry_get (self);
	return INT2FIX (toc->min);	
}

/*
 * Method: min=(min)
 * min: the 'min' property to set.
 *
 * Sets the 'min' property of the TOC entry.
 *
 * Returns: the 'min' property.
 */
static VALUE
rbburn_toc_entry_set_min (VALUE self, VALUE min)
{
	struct burn_toc_entry *toc = rbburn_toc_entry_get (self);
	toc->min = FIX2INT (min);
	return min;
}

/*
 * Method: sec
 *
 * Returns: the 'sec' property of the TOC entry.
 */
static VALUE
rbburn_toc_entry_get_sec (VALUE self)
{
	struct burn_toc_entry *toc = rbburn_toc_entry_get (self);
	return INT2FIX (toc->sec);	
}

/*
 * Method: sec=(sec)
 * sec: the 'sec' property to set.
 *
 * Sets the 'sec' property of the TOC entry.
 *
 * Returns: the 'sec' property.
 */
static VALUE
rbburn_toc_entry_set_sec (VALUE self, VALUE sec)
{
	struct burn_toc_entry *toc = rbburn_toc_entry_get (self);
	toc->sec = FIX2INT (sec);
	return sec;
}

/*
 * Method: frame
 *
 * Returns: the 'frame' property of the TOC entry.
 */
static VALUE
rbburn_toc_entry_get_frame (VALUE self)
{
	struct burn_toc_entry *toc = rbburn_toc_entry_get (self);
	return INT2FIX (toc->frame);	
}

/*
 * Method: frame=(frame)
 * frame: the 'frame' property to set.
 *
 * Sets the 'frame' property of the TOC entry.
 *
 * Returns: the 'frame' property.
 */
static VALUE
rbburn_toc_entry_set_frame (VALUE self, VALUE frame)
{
	struct burn_toc_entry *toc = rbburn_toc_entry_get (self);
	toc->frame = FIX2INT (frame);
	return frame;
}

/*
 * Method: zero
 *
 * Returns: the 'zero' property of the TOC entry.
 */
static VALUE
rbburn_toc_entry_get_zero (VALUE self)
{
	struct burn_toc_entry *toc = rbburn_toc_entry_get (self);
	return INT2FIX (toc->zero);	
}

/*
 * Method: zero=(zero)
 * zero: the 'zero' property to set.
 *
 * Sets the 'zero' property of the TOC entry.
 *
 * Returns: the 'zero' property.
 */
static VALUE
rbburn_toc_entry_set_zero (VALUE self, VALUE zero)
{
	struct burn_toc_entry *toc = rbburn_toc_entry_get (self);
	toc->zero = FIX2INT (zero);
	return zero;
}

/*
 * Method: pmin
 *
 * Returns: the 'pmin' property of the TOC entry.
 */
static VALUE
rbburn_toc_entry_get_pmin (VALUE self)
{
	struct burn_toc_entry *toc = rbburn_toc_entry_get (self);
	return INT2FIX (toc->pmin);	
}

/*
 * Method: pmin=(pmin)
 * pmin: the 'pmin' property to set.
 *
 * Sets the 'pmin' property of the TOC entry.
 *
 * Returns: the 'pmin' property.
 */
static VALUE
rbburn_toc_entry_set_pmin (VALUE self, VALUE pmin)
{
	struct burn_toc_entry *toc = rbburn_toc_entry_get (self);
	toc->pmin = FIX2INT (pmin);
	return pmin;
}

/*
 * Method: psec
 *
 * Returns: the 'psec' property of the TOC entry.
 */
static VALUE
rbburn_toc_entry_get_psec (VALUE self)
{
	struct burn_toc_entry *toc = rbburn_toc_entry_get (self);
	return INT2FIX (toc->psec);	
}

/*
 * Method: psec=(psec)
 * psec: the 'psec' property to set.
 *
 * Sets the 'psec' property of the TOC entry.
 *
 * Returns: the 'psec' property.
 */
static VALUE
rbburn_toc_entry_set_psec (VALUE self, VALUE psec)
{
	struct burn_toc_entry *toc = rbburn_toc_entry_get (self);
	toc->psec = FIX2INT (psec);
	return psec;
}

/*
 * Method: pframe
 *
 * Returns: the 'pframe' property of the TOC entry.
 */
static VALUE
rbburn_toc_entry_get_pframe (VALUE self)
{
	struct burn_toc_entry *toc = rbburn_toc_entry_get (self);
	return INT2FIX (toc->pframe);	
}

/*
 * Method: pframe=(pframe)
 * pframe: the 'pframe' property to set.
 *
 * Sets the 'pframe' property of the TOC entry.
 *
 * Returns: the 'pframe' property.
 */
static VALUE
rbburn_toc_entry_set_pframe (VALUE self, VALUE pframe)
{
	struct burn_toc_entry *toc = rbburn_toc_entry_get (self);
	toc->pframe = FIX2INT (pframe);
	return pframe;
}

void
rbburn_toc_entry_init (VALUE mBurn)
{
	VALUE c = rb_define_class_under (mBurn, "TocEntry", rb_cObject);
	rb_define_singleton_method (c, "new", rbburn_toc_entry_new, 0);
	rb_define_method (c, "session", rbburn_toc_entry_get_session, 0);
	rb_define_method (c, "adr", rbburn_toc_entry_get_adr, 0);
	rb_define_method (c, "control", rbburn_toc_entry_get_control, 0);
	rb_define_method (c, "tno", rbburn_toc_entry_get_tno, 0);
	rb_define_method (c, "point", rbburn_toc_entry_get_point, 0);
	rb_define_method (c, "min", rbburn_toc_entry_get_min, 0);
	rb_define_method (c, "sec", rbburn_toc_entry_get_sec, 0);
	rb_define_method (c, "frame", rbburn_toc_entry_get_frame, 0);
	rb_define_method (c, "zero", rbburn_toc_entry_get_zero, 0);
	rb_define_method (c, "pmin", rbburn_toc_entry_get_pmin, 0);
	rb_define_method (c, "psec", rbburn_toc_entry_get_psec, 0);
	rb_define_method (c, "pframe", rbburn_toc_entry_get_pframe, 0);
	rb_define_method (c, "session=", rbburn_toc_entry_set_session, 1);
	rb_define_method (c, "adr=", rbburn_toc_entry_set_adr, 1);
	rb_define_method (c, "control=", rbburn_toc_entry_set_control, 1);
	rb_define_method (c, "tno=", rbburn_toc_entry_set_tno, 1);
	rb_define_method (c, "point=", rbburn_toc_entry_set_point, 1);
	rb_define_method (c, "min=", rbburn_toc_entry_set_min, 1);
	rb_define_method (c, "sec=", rbburn_toc_entry_set_sec, 1);
	rb_define_method (c, "frame=", rbburn_toc_entry_set_frame, 1);
	rb_define_method (c, "zero=", rbburn_toc_entry_set_zero, 1);
	rb_define_method (c, "pmin=", rbburn_toc_entry_set_pmin, 1);
	rb_define_method (c, "psec=", rbburn_toc_entry_set_psec, 1);
	rb_define_method (c, "pframe=", rbburn_toc_entry_set_pframe, 1);
	cBurnTocEntry = c;
}
