/*
 *  Copyright (C) 2003, 2004 Laurent Sansonetti <lrz@gnome.org>
 *
 *  This file is part of Ruby/Libburn.
 *
 *  Ruby/Libburn is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  Ruby/Libburn is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Ruby/Libburn; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "rbburn.h"

/* Class: Burn::Track
 * This class references a single track on a disc.
 */
static VALUE cBurnTrack;

VALUE
rbburn_track_make (struct burn_track *c_struct)
{
	return c_struct != NULL
		? Data_Wrap_Struct (cBurnTrack,
				    NULL,
				    burn_track_free,
				    c_struct)
		: Qnil;
}

struct burn_track *
rbburn_track_get (VALUE obj)
{
	struct burn_track *c_struct;

	Data_Get_Struct(obj, struct burn_track, c_struct);
	assert (c_struct != NULL);
	return c_struct;
}

/*
 * Class method: new
 * 
 * Creates a track (for TAO recording, or to put in a session).
 *
 * Returns: a newly allocated Burn::Track object.
 */
static VALUE 
rbburn_track_new (VALUE self)
{
	return rbburn_track_make (burn_track_create ());
}

/*
 * Method: define_data(offset, tail, pad, mode)
 * offset: the library will write this many 0s before start of data.
 * tail: the number of extra 0s to write after data.
 * pad: true means the library should pad the last sector with 0s if the
 * track isn't exactly sector sized (otherwise the library will begin
 * reading from the next track).
 * mode: mode data format (bitfield).
 *
 * Defines the data in a track.
 *
 * Returns: self.
 */
static VALUE 
rbburn_track_define_data (VALUE self,
			  VALUE offset,
			  VALUE tail,
			  VALUE pad,
			  VALUE mode)
{
	burn_track_define_data (rbburn_track_get (self),
				FIX2INT (offset),
				FIX2INT (tail),
				RVAL2CBOOL (pad),
				FIX2INT (mode));
	return self;
}

/*
 * Method: set_isrc(country, owner, year, serial)
 * country: a 2 char country code, each character must be only numbers or
 * letters.
 * owner: a 3 char owner code, each character must be only numbers or
 * letters.
 * year: a 2 digit year (a number in 0-99, yep, not Y2K friendly).
 * serial: 5 digit serial number (a number in 0-99999).
 *
 * Sets the ISRC details for a track.
 * 
 * Returns: self.
 */
static VALUE
rbburn_track_set_isrc (VALUE self,
		       VALUE country,
		       VALUE owner,
		       VALUE year,
		       VALUE serial)
{
	burn_track_set_isrc (rbburn_track_get (self),
			     RVAL2CSTR (country),
			     RVAL2CSTR (owner),
			     NUM2CHR (year),
			     FIX2INT (serial));
	return self;
}

/*
 * Method: clear_isrc
 *
 * Disables ISRC parameters for a track.
 *
 * Returns: self.
 */
static VALUE
rbburn_track_clear_isrc (VALUE self)
{
	burn_track_clear_isrc (rbburn_track_get (self));
	return self;
}

/* Method: sectors
 * Returns: how long a track will be on disc.
 */
static VALUE
rbburn_track_get_sectors (VALUE self)
{
	return INT2FIX (burn_track_get_sectors (rbburn_track_get (self)));
}

/*
 * Method: set_source(source)
 * source: the data source to use for the contents of the track, as a
 * Burn::Source object.
 *
 * Sets the track's data source.
 *
 * Returns: an error code stating if the source is ready for use for
 * writing the track, or if an error occured (either Burn::Source::STATUS_OK,
 * Burn::Source::STATUS_EOF, Burn::Source::STATUS_FAILED).
 */
static VALUE
rbburn_track_set_source (VALUE self, VALUE source)
{
	return INT2FIX (burn_track_set_source (rbburn_track_get (self),
					       rbburn_source_get (source)));
}

/*
 * Method: entry
 *
 * Gets the associated TOC entry of this track.
 *
 * Returns: a Burn::TocEntry object.
 */
static VALUE
rbburn_track_get_entry (VALUE self)
{
	struct burn_toc_entry entry;
	burn_track_get_entry (rbburn_track_get (self), &entry);
	return rbburn_toc_entry_make (&entry);
}

/*
 * Method: mode
 *
 * Gets the audio/data mode for the entry. Derived from control and
 * possibly from reading the track's first sector.
 *
 * Returns: the mode of the track.
 */
static VALUE
rbburn_track_get_mode (VALUE self)
{
	return INT2FIX (burn_track_get_mode (rbburn_track_get (self)));
}

void
rbburn_track_init (VALUE mBurn)
{
	VALUE c = rb_define_class_under (mBurn, "Track", rb_cObject);
	rb_define_singleton_method (c, "new", rbburn_track_new, 0);
	rb_define_method (c, "define_data", rbburn_track_define_data, 4);
	rb_define_method (c, "set_isrc", rbburn_track_set_isrc, 4);
	rb_define_method (c, "clear_isrc", rbburn_track_clear_isrc, 0);
	rb_define_method (c, "sectors", rbburn_track_get_sectors, 0);
	rb_define_method (c, "set_source", rbburn_track_set_source, 1);
	rb_define_method (c, "entry", rbburn_track_get_entry, 0);
	rb_define_alias (c, "toc_entry", "entry");
	rb_define_method (c, "mode", rbburn_track_get_mode, 0);
	cBurnTrack = c;
}
