/*
 *  Copyright (C) 2003, 2004 Laurent Sansonetti <lrz@gnome.org>
 *
 *  This file is part of Ruby/Libburn.
 *
 *  Ruby/Libburn is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  Ruby/Libburn is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Ruby/Libburn; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "rbburn.h"

/* Class: Burn::WriteOptions
 * Options relative to writing operations.
 */
static VALUE cBurnWriteOpts;

static VALUE
rbburn_write_opts_make (struct burn_write_opts *c_struct)
{
	return c_struct != NULL
		? Data_Wrap_Struct (cBurnWriteOpts,
				    NULL,
				    burn_write_opts_free,
				    c_struct)
		: Qnil;
}

struct burn_write_opts *
rbburn_write_opts_get (VALUE obj)
{
	struct burn_write_opts *c_struct;

	Data_Get_Struct (obj, struct burn_write_opts, c_struct);
	assert (c_struct != NULL);
	return c_struct;
}

/*
 * Class method: new(drive)
 * drive: the drive to write with (as a Burn::Drive).
 *
 * Creates a new Burn::WriteOptions for burning to the specified drive.
 *
 * Returns: a newly allocated Burn::WriteOptions object.
 */
static VALUE
rbburn_write_opts_new (VALUE self, VALUE drive)
{
	struct burn_drive_info *info = rbburn_drive_get (drive);
	return rbburn_write_opts_make (burn_write_opts_new (info->drive));
}

/*
 * Method: set_write_type(write_type, block_type)
 * write_type: the write type to use (either Burn::WriteOptions::TYPE_PACKET,
 * Burn::WriteOptions::TYPE_TAO, Burn::WriteOptions::TYPE_SAO,
 * Burn::WriteOptions::TYPE_RAW).
 * block_type: the block type to use (either Burn::BLOCK_RAW0,
 * Burn::BLOCK_RAW16, Burn::BLOCK_RAW96P, Burn::BLOCK_RAW96R,
 * Burn::BLOCK_MODE1, Burn::BLOCK_MODE2_PATHETIC, Burn::BLOCK_MODE2_LAME,
 * Burn::BLOCK_MODE2_OBSCURE, Burn::BLOCK_MODE2_OK, Burn::BLOCK_MODE2R).
 *
 * Sets the write type.
 *
 * Returns: true on success, false otherwise.
 */
static VALUE
rbburn_write_opts_set_write_type (VALUE self,
				  VALUE write_type,
				  VALUE block_type)
{
	return CBOOL2RVAL (burn_write_opts_set_write_type (
				rbburn_write_opts_get (self),
				FIX2INT (write_type),
				FIX2INT (block_type)));
}

/*
 * Method: set_toc_entries(*toc_entries)
 * toc_entries: TOC entries (as a list of Burn::TocEntry objects).
 *
 * Supplies toc entries for writing - not normally required for cd mastering.
 *
 * Returns: self.
 */
static VALUE
rbburn_write_opts_set_toc_entries (VALUE self,
				   VALUE toc_entries)
{
	struct burn_toc_entry *entries[128];
	int i;

	if (RARRAY (toc_entries)->len > sizeof (entries))
		rb_raise(rb_eRuntimeError, "Too much TOC entries.");

	for (i = 0; i < RARRAY (toc_entries)->len; i++) {
		entries[i] = rbburn_toc_entry_get (rb_ary_entry (toc_entries,
								 i));
	}
	burn_write_opts_set_toc_entries (rbburn_write_opts_get (self),
					 RARRAY (toc_entries)->len,
					 *entries);
	return self;
}

/*
 * Method: set_simulate(state)
 * state: if true, the drive will perform a simulation instead of a burn.
 *
 * Sets wheither the drive should perform a simulation instead of a burn or
 * not.
 *
 * Returns: true on success, false otherwise.
 */
static VALUE
rbburn_write_opts_set_simulate (VALUE self, VALUE state)
{
	return CBOOL2RVAL (burn_write_opts_set_simulate (
				rbburn_write_opts_get (self),
				RVAL2CBOOL (state)));
}

/*
 * Method: set_underrun_proof(state)
 * state: if true, buffer underrun protection is enabled. 
 *
 * Controls buffer underrun prevention.
 *
 * Returns: true on success, false otherwise.
 */
static VALUE
rbburn_write_opts_set_underrun_proof (VALUE self, VALUE state)
{
	return CBOOL2RVAL (burn_write_opts_set_underrun_proof (
				rbburn_write_opts_get (self),
				RVAL2CBOOL (state)));
}

/*
 * Method: set_perform_opc(state)
 * state: if true, optical power calibration will be performed at
 * start of burn.
 *
 * Sets whether the drive should use opc or not.
 *
 * Returns: self.
 */
static VALUE
rbburn_write_opts_set_perform_opc (VALUE self, VALUE state)
{
	burn_write_opts_set_perform_opc (rbburn_write_opts_get (self),
					 RVAL2CBOOL (state));
	return self;
}

void
rbburn_write_opts_init (VALUE mBurn)
{
	VALUE c = rb_define_class_under (mBurn, "WriteOptions", rb_cObject);
	rb_define_singleton_method (c, "new", rbburn_write_opts_new, 1);
	rb_define_method (c, "set_write_type",
			  rbburn_write_opts_set_write_type, 2);
	rb_define_method (c, "set_toc_entries",
			  rbburn_write_opts_set_toc_entries, -2);
	rb_define_method (c, "set_simulate",
			  rbburn_write_opts_set_simulate, 1);
	rb_define_method (c, "set_underrun_proof",
			  rbburn_write_opts_set_underrun_proof, 1);
	rb_define_method (c, "set_perform_opc",
			  rbburn_write_opts_set_perform_opc, 1);

	/* Constant: TYPE_PACKET
	 * Packet writing.
	 */
	rb_define_const (c, "TYPE_PACKET", BURN_WRITE_PACKET);

	/* Constant: TYPE_TAO
	 * Track At Once recording. 2s gaps between tracks, no fonky lead-ins.
	 */
	rb_define_const (c, "TYPE_TAO", BURN_WRITE_TAO);

	/* Constant: TYPE_SAO
	 * Session At Once.
	 */
	rb_define_const (c, "TYPE_SAO", BURN_WRITE_SAO);

	/* Constant: TYPE_RAW
	 * Raw disc at once recording. All subcodes must be provided by lib
	 * or user. Only raw block types are supported.
	 */
	rb_define_const (c, "TYPE_RAW", BURN_WRITE_RAW);
	cBurnWriteOpts = c;
}
